open Alcotest
open Base
open Okweb
open Lwt.Infix

let test_smoke_test_of_server_and_client =
  test_case "smoke test" `Quick begin fun () ->
    let handler req =
      let header_value =
        req
        |> Request.find_header ~key:"X-Test-Request"
        |> Option.value ~default:"INVALID" in
      (req |> Request.get_body |> Body.read_up_to) >>= fun body ->
      Lwt.return Response.(
          ok
          |> with_header ~key:"X-Test-Response" ~value:header_value
          |> with_string_body body)
    in

    let port = 54321 in
    let uri = Printf.sprintf "http://localhost:%d/" port in
    let req = Request.(
        post uri
        |> with_header ~key:"X-Test-Request" ~value:"PASS"
        |> with_string_body "OK") in
    let (resp, body) = Lwt_main.run begin
        Lwt.async (fun () -> Server.listen ~address:"localhost" ~port handler);
        Client.send req >>= fun resp ->
        Body.read_up_to (Response.get_body resp) >>= fun body ->
        Lwt.return (resp, body)
      end in

    check (option string)
      "invalid test header"
      (Some "PASS")
      (Response.find_header ~key:"X-Test-Response" resp);
    check string
      "invalid body"
      "OK"
      body;
  end


let cases = [
  (* FIXME: investigate why it fails on CI with ECONNREFUSED *)
  (* test_smoke_test_of_server_and_client; *)
]
